#include <vtkCocoaRenderWindow.h>
#include <vtkCocoaRenderWindowInteractor.h>
#include <vtkRenderer.h>
#include <vtkLODActor.h>
#include <vtkPolyDataMapper.h>
#include <vtkAxes.h>
#include <vtkVectorText.h>
#include <vtkTextActor.h>
#include <vtkSmartPointer.h>
#include <vtkTextProperty.h>
#include <vtkSphereSource.h>
#include <vtkProperty.h>
#include <vtkFollower.h>

#define VTK_NEW(type, name) vtkSmartPointer<type> name = vtkSmartPointer<type>::New()

int main(void)
{
    VTK_NEW(vtkCocoaRenderWindow, win);
    VTK_NEW(vtkCocoaRenderWindowInteractor, iren);
    VTK_NEW(vtkRenderer, ren);
    ren->SetBackground(0, 0.5, 0.5);
    iren->SetRenderWindow(win);
    win->AddRenderer(ren);
    iren->Initialize();

    /* Axes */
    VTK_NEW(vtkAxes, axes);
    VTK_NEW(vtkPolyDataMapper, axesMapper);
    VTK_NEW(vtkActor, axesActor);
    axesMapper->SetInputConnection(axes->GetOutputPort());
    axesActor->SetMapper(axesMapper);

    /* 2D text */
    VTK_NEW(vtkTextActor, text2d);
    text2d->SetInput("I am a 2D text");
    text2d->GetPositionCoordinate()->SetCoordinateSystemToWorld();
    text2d->SetPosition(0.2, 0.2);
    text2d->GetTextProperty()->SetFontSize(18);

    /* X, Y, Z */
    VTK_NEW(vtkTextActor, x2d);
    x2d->SetInput("X");
    x2d->GetPositionCoordinate()->SetCoordinateSystemToWorld();
    x2d->SetPosition(1, 0);
    x2d->GetTextProperty()->SetFontSize(24);
    VTK_NEW(vtkTextActor, y2d);
    y2d->SetInput("Y");
    y2d->GetPositionCoordinate()->SetCoordinateSystemToWorld();
    y2d->SetPosition(0, 1);
    y2d->GetTextProperty()->SetFontSize(24);
    VTK_NEW(vtkTextActor, z2d);
    z2d->SetInput("Z");
    z2d->GetPositionCoordinate()->SetCoordinateSystemToWorld();
    z2d->SetPosition(0, 0);
    z2d->GetTextProperty()->SetFontSize(24);

    /* sphere */
    VTK_NEW(vtkSphereSource, sphere);
    VTK_NEW(vtkPolyDataMapper, sphereMapper);
    VTK_NEW(vtkActor, sphereActor);
    sphere->SetCenter(0.4, 0.3, 0.1);
    sphere->SetRadius(0.5);
    sphereMapper->SetInputConnection(sphere->GetOutputPort());
    sphereActor->SetMapper(sphereMapper);
    sphereActor->GetProperty()->SetColor(0, 1, 0);

    /* 3D text */
    VTK_NEW(vtkVectorText, text3d);
    VTK_NEW(vtkPolyDataMapper, text3dMapper);
    VTK_NEW(vtkFollower, text3dFollower);
    text3d->SetText("I am a 3D vectorized text");
    text3dMapper->SetInputConnection(text3d->GetOutputPort());
    text3dFollower->SetMapper(text3dMapper);
    text3dFollower->SetScale(0.2, 0.2, 0.2);
    text3dFollower->AddPosition(0.5, 0.3, 1);
    text3dFollower->SetCamera(ren->GetActiveCamera());

    ren->AddViewProp(axesActor);
    ren->AddViewProp(text2d);
    ren->AddViewProp(x2d);
    ren->AddViewProp(y2d);
    ren->AddViewProp(z2d);
    ren->AddViewProp(sphereActor);
    ren->AddViewProp(text3dFollower);

    win->Render();
    iren->Start();
}
